using System;
using System.Collections.Generic;
using System.Text;
using System.Collections.ObjectModel;
using System.ComponentModel;

namespace WaveData
{
    class HarmonicSeries : INotifyPropertyChanged
    {
        #region Fields

        /// <summary>
        /// Value of Amplitudes property.
        /// </summary>
        private Amplitude[] amplitudeValues;

        /// <summary>
        /// Value of Outputs property
        /// </summary>
        private double[] outputValues;

        #endregion

        #region Constructor

        /// <summary>
        /// Creates a new harmonic series generator.
        /// </summary>
        /// <param name="numberOfHarmonics">The number of harmonics to hold
        /// in the Amplitudes property.</param>
        /// <param name="numberOfOutputValues">The number of output points
        /// to generate in the Outputs property.</param>
        public HarmonicSeries(int numberOfHarmonics, int numberOfOutputValues)
        {
            amplitudeValues = new Amplitude[numberOfHarmonics];
            outputValues = new double[numberOfOutputValues];

            for (int i = 0; i < amplitudeValues.Length; ++i)
            {
                amplitudeValues[i] = new Amplitude();
                amplitudeValues[i].PropertyChanged += new PropertyChangedEventHandler(HarmonicSeries_PropertyChanged);
            }
            amplitudeValues[0].Value = 1.0;
        }

        #endregion

        #region Change handling

        /// <summary>
        /// Called by items in the Amplitudes array to notify us that an
        /// amplitude value has changed.
        /// </summary>
        void HarmonicSeries_PropertyChanged(object sender, PropertyChangedEventArgs e)
        {
            OnPropertyChanged("Amplitudes");
            UpdateValues();
        }

        #endregion

        #region Properties

        /// <summary>
        /// The amplitudes of the harmonics in the series.
        /// </summary>
        /// <remarks>When changing values in this array, change the
        /// Value property of the Amplitude. Do not replace the array
        /// entry with a new Amplitude object.</remarks>
        public Amplitude[] Amplitudes
        {
            get { return amplitudeValues; }
        }


        /// <summary>
        /// The output waveform, generated from the current values in
        /// the Amplitudes collection.
        /// </summary>
        public double[] Output
        {
            get { return outputValues; }
        }

        #endregion

        #region Events
        
        /// <summary>
        /// Raised whenever a property value changes.
        /// </summary>
        public event PropertyChangedEventHandler PropertyChanged;

        #endregion

        #region Private methods


        /// <summary>
        /// Recalculate the Outputs based on the current Amplitudes.
        /// </summary>
        private void UpdateValues()
        {
            for (int point = 0; point < outputValues.Length; ++point)
            {
                double theta = point * 2 * Math.PI;
                theta /= outputValues.Length;

                double value = 0;
                for (int harmonic = 0; harmonic < amplitudeValues.Length; ++harmonic)
                {
                    value += amplitudeValues[harmonic].Value *
                        Math.Sin(theta * (harmonic + 1));
                }
                outputValues[point] = value;
            }

            OnPropertyChanged("Output");
        }


        /// <summary>
        /// Raise a property change notification.
        /// </summary>
        /// <param name="propertyName">Name of the property that changed.</param>
        private void OnPropertyChanged(string propertyName)
        {
            if (PropertyChanged != null)
            {
                PropertyChanged(this, new PropertyChangedEventArgs(propertyName));
            }
        }

        #endregion

    }
}
